# this is the file with the lmer multi-level rddAnalysis
library(tidyverse)
library(plyr) 
# 0 loading the readme data in 
try(setwd(dirname(rstudioapi::getActiveDocumentContext()$path)))
readme_df <- read_csv("../final_data/deb_readme_did.csv")
# 1 preprocessing 
#colnames(readme_df) <- c("upstream_vcs_link", "event_date", "event_hash", "before_all_ct", "before_mrg_ct", "after_all_ct", "after_mrg_ct", "before_auth_new", "after_commit_new", "after_auth_new", "before_commit_new")
col_order <- c("upstream_vcs_link", "age_of_project", "event_date", "event_hash", "before_all_ct", "after_all_ct", "before_mrg_ct", "after_mrg_ct", "before_auth_new", "after_auth_new", "before_commit_new",  "after_commit_new")
readme_df <- readme_df[,col_order]
readme_df$ct_before_all <- str_split(gsub("[][]","", readme_df$before_all_ct), ", ")
readme_df$ct_after_all <- str_split(gsub("[][]","", readme_df$after_all_ct), ", ")
readme_df$ct_before_mrg <- str_split(gsub("[][]","", readme_df$before_mrg_ct), ", ")
readme_df$ct_after_mrg <- str_split(gsub("[][]","", readme_df$after_mrg_ct), ", ")
drop <- c("before_all_ct", "before_mrg_ct", "after_all_ct", "after_mrg_ct")
readme_df = readme_df[,!(names(readme_df) %in% drop)]
# 2 some expansion needs to happens for each project 
expand_timeseries <- function(project_row) {
  longer <- project_row |> 
    pivot_longer(cols = starts_with("ct"),
                 names_to = "window",
                 values_to = "count") |>
    unnest(count) 
  longer$observation_type <- gsub("^.*_", "", longer$window)
  longer <- ddply(longer, "observation_type", transform, week=seq(from=0, by=1, length.out=length(observation_type)))
  longer$count <- as.numeric(longer$count)
  #longer <- longer[which(longer$observation_type == "all"),]
  return(longer)
}
expanded_data <- expand_timeseries(readme_df[1,])
for (i in 2:nrow(readme_df)){
  expanded_data <- rbind(expanded_data, expand_timeseries(readme_df[i,]))
}
#filter out the windows of time that we're looking at
window_num <- 8
windowed_data <- expanded_data |>
  filter(week >= (27 - window_num) & week <= (27 + window_num)) |>
  mutate(D = ifelse(week > 27, 1, 0))
#scale the age numbers 
windowed_data$scaled_project_age <- scale(windowed_data$age_of_project)
windowed_data$week_offset <- windowed_data$week - 27
#separate out the cleaning d
all_actions_data <- windowed_data[which(windowed_data$observation_type == "all"),]
mrg_actions_data <- windowed_data[which(windowed_data$observation_type == "mrg"),]
#find some EDA to identify which types of models might be the best for this
hist(log(all_actions_data$count))
median(all_actions_data$count)
table(all_actions_data$count)
var(all_actions_data$count)
qqnorm(all_actions_data$count)
y <- qunif(ppoints(length(all_actions_data$count)))
qqplot(all_actions_data$count, y)
all_actions_data$logged_count <- log(all_actions_data$count)
all_actions_data$log1p_count <- log1p(all_actions_data$count)
# 3 rdd in lmer analysis
# rdd: https://rpubs.com/phle/r_tutorial_regression_discontinuity_design
# lmer: https://www.youtube.com/watch?v=LzAwEKrn2Mc
library(lme4)
# https://www.bristol.ac.uk/cmm/learning/videos/random-intercepts.html#exvar
library(optimx)
library(lattice)
all_model <- lmer(log1p_count ~ D * I(week_offset)+ scaled_project_age + (D * I(week_offset)| upstream_vcs_link), data=all_actions_data, REML=FALSE, control = lmerControl(
  optimizer ='optimx', optCtrl=list(method='L-BFGS-B')))
summary_of_all <- summary(all_model)
#identifying the quartiles of effect for D
all_model_ranef_condvar <- ranef(all_model, condVar = TRUE)
all_model_ranef <- ranef(all_model, condVar = FALSE)
attr(all_model_ranef_condvar$upstream_vcs_link, "postVar")

all_coefficients <- coef(all_model)
all_standard_errors <- sqrt(diag(vcov(all_model)))[1]
#all_conf_intervals <- cbind(all_coefficients - 1.96 * all_standard_errors, 
#                        all_coefficients + 1.96 * all_standard_errors)

df_ranefs <- as.data.frame(all_model_ranef_condvar)
df_rn_no_cv <- as.data.frame(all_model_ranef)
D_df_ranef <- df_ranefs[which(df_ranefs$term == "D"),]
#below this groups the ranefs
"""
has_zero <- function(condval, condsd){
  bounds <- condsd * 1.96
  return(ifelse(((condval - bounds) < 0),ifelse(((condval + bounds) > 0), 1, 0), 2))
}
df_ranefs <- df_ranefs |>
    mutate(ranef_grouping = has_zero(condval, condsd)) |>
    mutate(rank = rank(condval))
D_df_ranef <- df_ranefs[which(df_ranefs$term == ),]
D_df_ranef <- D_df_ranef |>
  mutate(rank = rank(condval))
hist(D_df_ranef$ranef_grouping)
#plot the ranefs 
library(ggplot2)
D_df_ranef |>
  ggplot(aes(x=rank, y=condval, col = as.factor(ranef_grouping))) +
  geom_linerange(aes(ymin= condval - (1.96 * condsd), ymax= condval + (1.96 * condsd))) +
  theme_bw()
"""
#d_effect_ranef_all <- all_model_ranef$upstream_vcs_link
#d_effect_ranef_all$quartile <- ntile(d_effect_ranef_all$condval, 4)
#model residuals
all_residuals <- residuals(all_model)
qqnorm(all_residuals)
# mrg behavior for this
mrg_model <- lmer(log1p_count ~ D * I(week_offset)+ scaled_project_age + (D * I(week_offset)| upstream_vcs_link), data=all_actions_data, REML=FALSE, control = lmerControl(
  optimizer ='optimx', optCtrl=list(method='L-BFGS-B')))
summary(mrg_model)
#identifying the quartiles of effect for D
mrg_model_ranef <- ranef(mrg_model, condVar=TRUE)
df_mrg_ranefs <- as.data.frame(mrg_model_ranef)
dotplot(mrg_model_ranef)
d_effect_ranef_mrg <- mrg_model_ranef[mrg_model_ranef$term=="D",]
d_effect_ranef_mrg$quartile <- ntile(d_effect_ranef_mrg$condval, 4)
#doing similar random effect analysis for this
df_mrg_ranefs <- df_mrg_ranefs |>
  mutate(ranef_grouping = has_zero(condval, condsd)) |>
  mutate(rank = rank(condval))
D_df_mrg_ranefs <- df_mrg_ranefs[which(df_mrg_ranefs$term == "D"),]
D_df_mrg_ranefs  |>
  ggplot(aes(x=rank, y=condval, col = as.factor(ranef_grouping))) +
  geom_linerange(aes(ymin= condval - (1.96 * condsd), ymax= condval + (1.96 * condsd))) 
#merge model residuals
mrg_residuals <- residuals(mrg_model)
qqnorm(mrg_residuals)
# Performance: 